"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getGithubLinks = exports.getGithubLink = void 0;
const cp = require("child_process");
const path = require("path");
const url = require("url");
/**
 * Returns a GitHub URL (or a file URL, if the file is untracked).
 * Throws if no appropriate remote can be identified.
 */
async function getGithubLink(path, lineNumber) {
    return (await getGithubLinkWorker([{ path, lineNumber }]))[0];
}
exports.getGithubLink = getGithubLink;
/**
 * Returns a GitHub URL (or a file URL, if untracked) for each location.
 * Throws if no appropriate remote can be identified.
 */
async function getGithubLinks(locations) {
    return await getGithubLinkWorker(locations);
}
exports.getGithubLinks = getGithubLinks;
async function getGithubLinkWorker(locations) {
    if (!locations.length) {
        return [];
    }
    const cwd = path.dirname(locations[0].path);
    // This is a clever way to quickly retrieve the current commit
    const commit = await getExecOutput("git", ["rev-parse", "@"], cwd);
    if (!commit) {
        throw new Error(`Couldn't identify commit - not a repository?`);
    }
    const preferredRemote = await getPreferredRemote(cwd, commit);
    if (!preferredRemote) {
        throw new Error(`Commit ${commit} is not present on any remote`);
    }
    const repoUrl = (await getExecOutput("git", ["remote", "get-url", `${preferredRemote}`], cwd)).replace(/\.git$/, "");
    // In practice, it's common to see many requests for (different lines in) the same file
    const serverPathCache = new Map(); // local to server
    const urls = [];
    for (const location of locations) {
        const localPath = path.resolve(location.path);
        const lineNumber = location.lineNumber;
        // We would just use path math, but this will also respect git's casing
        let serverPath = serverPathCache.get(localPath);
        if (!serverPath) {
            serverPath = await getExecOutput("git", ["ls-files", "--full-name", "--", `${localPath}`], cwd);
            serverPathCache.set(localPath, serverPath);
        }
        // Use a file URL if the file is untracked
        const fileUrl = serverPath
            ? `${repoUrl}/blob/${commit}/${serverPath}`
            : url.pathToFileURL(localPath).toString();
        // Cheat and add line numbers to file URLs too - VS Code handles it
        urls.push(lineNumber ? `${fileUrl}#L${lineNumber}` : fileUrl);
    }
    return urls;
}
async function getPreferredRemote(cwd, commit) {
    let containingRemotes = [];
    const refsRegex = /^refs\/remotes\/([^\/]+)/gm;
    const refs = await getExecOutput("git", ["for-each-ref", `--format=%(refname)`, "--contains", commit, "refs/remotes"], cwd);
    let refMatch;
    while (refMatch = refsRegex.exec(refs)) {
        containingRemotes.push(refMatch[1]);
    }
    if (containingRemotes.length) {
        return containingRemotes.find(r => r === "origin") || containingRemotes[0];
    }
    // Sometimes, the for-each-ref trick doesn't work (e.g. in some submodules), so we fall back on a slower method
    // Sort `origin` to the front, if it's present
    const allRemotes = (await getExecOutput("git", ["remote"], cwd)).split(/\r\n?|\n/).sort((a, b) => +(b === "origin") - +(a === "origin"));
    for (const remote of allRemotes) {
        const status = await getSpawnExitCode("git", ["fetch", "--dry-run", "--quiet", remote, commit], cwd);
        if (status === 0) {
            return remote;
        }
    }
    return undefined;
}
function getExecOutput(command, args, cwd) {
    return new Promise(resolve => {
        cp.execFile(command, args, { cwd, encoding: "utf-8" }, (err, stdout, stderr) => {
            resolve((err || stderr) ? "" : stdout.trim());
        });
    });
}
function getSpawnExitCode(command, args, cwd) {
    return new Promise(resolve => {
        const proc = cp.spawn(command, args, { cwd, stdio: "ignore" });
        proc.on("close", code => resolve(code));
    });
}
//# sourceMappingURL=index.js.map